import os, sys, argparse, struct

tracks=[]

#########################################################
def readmtc(fname):
  global tracks
  tracks=[]
  print("Loading MTC:",fname)
  with open(fname, "rb") as f:
    dat=f.read()
  fsize=os.path.getsize(fname)
  header=struct.unpack(">4sI", dat[0:8])

  if header[0]!=b"MTC1":
    print('Not an .MTC file format')
    return False
  n=8
  while True:
    print(dat[n:n+8])
    header=struct.unpack(">4sI", dat[n:n+8])
    n+=8
    m=n+header[1]
    data=dat[n:m]
    if header[0]==b"TRCK":
      header2=struct.unpack(">4sI", data[0:8])
      data2=data[8:8+header2[1]]
      tracks.append(data2)
     
    if not header[0] in (b"PROP",b"TRCK",b"NAME"): m+=1
    n=m
    if n==fsize: break
  if len(tracks)!=3:
    print('Not 3TS module')
    return False
  return True

#########################################################
def savemtc(fname):
  global tracks
  print("Saving MTC:",fname)
  num=len(tracks)
  data=bytearray()

  leng=8
  for i in range(num):
    leng+=len(tracks[i])+8+8+14

  data+=struct.pack(">4sI", b"MTC1", leng)

  for i in range(num):
    data+=struct.pack(">4sI", b"TRCK", len(tracks[i])+8+8+14)
    data+=struct.pack(">4sI", b"DATA", len(tracks[i]))
    data+=tracks[i]
    data+=struct.pack(">4sI", b"PROP", 14)
    data+=bytes('Filename='+str(i+1)+'.pt3','UTF-8')

  with open(fname, "wb") as f:
    f.write(data)

#########################################################
def readpt3(fname):
  global tracks
  tracks=[]
  print("Loading PT3:",fname)
  with open(fname, "rb") as f:
    dat=f.read()
  fsize=os.path.getsize(fname)
  suffix=dat[-4:]

  if suffix!=b'03TS':
    print('Not TS3 module')
    return False

  suffix=struct.unpack("<4sH4sH4sH", dat[-22:-4])
  n=0
  for i in range(3):
    if suffix[i*2]==b'PT3!':
      m=suffix[i*2+1]
      tracks.append(dat[n:n+m])
      n+=m
    else:
      print('Wrong sub-module type:',suffix[i*2])
      return False
  if len(tracks)!=3:
    print('Not 3TS module')
    return False
  return True

#########################################################
def savept3(fname):
  global tracks
  print("Saving PT3:",fname)
  data=bytearray()
  for i in range(3):
    data+=tracks[i]

  suff=bytearray()
  for i in range(3):
    suffix=struct.pack("<4sH", b"PT3!", len(tracks[i]))
    suff+=suffix
  suff+=b'03TS'

  with open(fname, "wb") as f:
    f.write(data)
    f.write(suff)

#########################################################
def _main(argv):
  print('PT3(TS3)<->MTC(ZXTune) converter by Dexus(Volutar)')
  parser = argparse.ArgumentParser(
    formatter_class=argparse.ArgumentDefaultsHelpFormatter)
  parser.add_argument("infile", help="in filename")
  parser.add_argument("outfile", nargs='?', help="out filename")
  args = parser.parse_args(argv[1:])

  if args.outfile==None:
    other=os.path.splitext(args.infile)[1]
    if other.upper()=='.PT3': other='.mtc'
    elif other.upper()=='.MTC': other='.pt3'
    else:
      print('Unprocessable file extension')
      return
    outfile=os.path.splitext(args.infile)[0]+other
  else: outfile=args.outfile

  ext1=os.path.splitext(args.infile)[1].upper()
  ext2=os.path.splitext(outfile)[1].upper()
  if ext1=='.MTC': l=readmtc(args.infile)
  if ext1=='.PT3': l=readpt3(args.infile)
  if not l: return;
  if ext2=='.PT3': savept3(outfile)
  if ext2=='.MTC': savemtc(outfile)

if __name__ == "__main__":
  _main(sys.argv)
